//===-- tapi/tapi.h - TAPI C++ Library Interface ----------------*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
///
/// \file
/// \brief This is the umbrella header for the TAPI C++ Library Interface.
/// \since 1.0
///
//===----------------------------------------------------------------------===//
#ifndef TAPI_H
#define TAPI_H

///
/// \defgroup TAPI_CPP_API TAPI C++ API
///
/// The C++ Application Programming Interface (API) for the TAPI library
///

#include <tapi/APIVersion.h>
#include <tapi/Defines.h>
#include <tapi/LinkerInterfaceFile.h>
#include <tapi/PackedVersion32.h>
#include <tapi/Symbol.h>
#include <tapi/Version.h>

#endif // TAPI_H
